package com.sundata.internalevaluation.calc.calcUnit;

import cn.hutool.core.util.StrUtil;
import com.sundata.common.util.DBExecutor;
import com.sundata.internalevaluation.calc.model.CalcException;
import com.sundata.internalevaluation.calc.model.CalcResult;
import com.sundata.internalevaluation.calc.model.CalcUnit;
import com.sundata.internalevaluation.calc.model.finals.CalcType;
import com.sundata.internalevaluation.configuration.model.IndexConfigModel;
import com.sundata.internalevaluation.configuration.model.IndexSourceModel;
import com.sundata.internalevaluation.configuration.service.IndexConfigService;
import com.sundata.internalevaluation.script.ScriptUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;

public class IndexCalcUnit extends CalcUnit {

    private IndexConfigService indexConfigService;

    private IndexConfigModel indexConfigModel;

    private static final Logger log = LoggerFactory.getLogger(IndexCalcUnit.class);
    /**
     * 创建数据单元的绝对对象，对象必须包含如下参数
     *
     * @param calcCode    计算对象编号
     * @param calcName    计算对象名称
     * @param initContext 计算单元初始化参数
     */
    public IndexCalcUnit(String calcCode, String calcName, Map<String, Object> initContext, IndexConfigModel indexConfigModel,IndexConfigService indexConfigService) {
        super(calcCode, calcName, CalcType.INDEX, initContext);
        this.indexConfigModel = indexConfigModel;
        this.indexConfigService = indexConfigService;
    }

    /**
     * 判断是否已经计算过数据了
     *
     * @param calculateInstanceNumber 计算流水号
     * @return 是否计算过 true 计算过 false 没有计算过
     */
    @Override
    public boolean isCalcFinished(String calculateInstanceNumber) {
        // TODO 计算是否已经计算
        return false;
//        return DataImages.indexCalcUnitHashMap.containsKey(calculateInstanceNumber) && DataImages.indexCalcUnitHashMap.get(calculateInstanceNumber).stream().anyMatch(a -> a.getCalcCode().equals(this.getCalcCode()));
    }

    /**
     * 初始化计算结果的方法，如果已经计算过，在实现过程中，应当在此方法中根据计算流水号重新初始化 resultContext 结果对象，为其他依赖对象做准备
     * 若明明计算过本单元但再次计算时没有初始化该对象，则计算依赖出现问题无法定位与处理
     *
     * @param calculateInstanceNumber 计算流水号
     */
    @Override
    public void initResultContext(String calculateInstanceNumber) {
        // TODO 初始化
        List<IndexCalcUnit> indexCalcUnits = new ArrayList<>();
//        List<IndexCalcUnit> indexCalcUnits = DataImages.indexCalcUnitHashMap.get(calculateInstanceNumber);
        // 筛选并查找对象，如果找不到则报错
        if (indexCalcUnits.stream().noneMatch(a -> a.getCalcCode().equals(this.getCalcCode()))) {
            throw new CalcException(calculateInstanceNumber, StrUtil.format("无法找到已计算完成的结果，计算单元编号：{}，计算流水号为：{}。", this.getCalcCode(), calculateInstanceNumber));
        }
        this.setResultContext(indexCalcUnits.stream().filter(a -> a.getCalcCode().equals(this.getCalcCode())).findFirst().get().getResultContext());
    }

    /**
     * 根据节点配置获取源节点；
     *
     * @return 所有源头节点
     */
    @Override
    public List<CalcUnit> getSourceCalcUnits() {
        // TODO 获取源头节点
        List<IndexSourceModel> indexList = indexConfigService.getIndexSourceList(new IndexConfigModel());
        indexList.stream().map(sourceModel -> new IndexSourceCalcUnit
                (sourceModel.getOtherIndexNo(),"",CalcType.INDEX,Map.of(),sourceModel));
        return new ArrayList<>();
//        return ConfigImages.indexCalcUnitListMap.get(this);
    }


    /**
     * 计算之后的方法，可实现为空
     *
     * @param context 计算参数过程数据
     */
    @Override
    public void afterCalc(Map<String, Object> context) {
        log.debug("计算之后的参数结构：{}",context);
    }

    /**
     * 计算之前，可实现空
     *
     * @param context 计算参数过程数据
     */
    @Override
    public void beforeCalc(Map<String, Object> context) {
        log.debug("计算之前的参数结构：{}",context);
    }



    /**
     * 必须实现的主体计算内容
     *
     * @param context 节点计算参数清单
     * @param sourceResults 整个计算过程中的节点结果
     */
    @Override
    public void calc(final CalcResult<String, Object> thisResult, String calculateInstanceNumber, Map<String, Object> context, Map<CalcUnit, CalcResult<String, Object>> sourceResults) {

        // TODO 实际的计算过程

        AtomicInteger i = new AtomicInteger();
        AtomicInteger j = new AtomicInteger();
        CalcResult<String, Object> indexResult = null;
        CalcResult<String, Object> dataSetResult = null;
        // List<IndexSourceModel> sourceModelList = indexConfigModel.getIndexSourceModel();
        sourceResults.forEach((calcUnit,result) -> {
            // 数据集
            if ( calcUnit instanceof DataSetCalcUnit dataSetCalcUnit){
                if(dataSetCalcUnit.getCalcType() == CalcType.DATASET){
                    i.getAndIncrement();
                    dataSetResult.putAll(result);
                }
            }
            // 其他指标
            if( calcUnit instanceof IndexCalcUnit indexCalcUnit){
                if(indexCalcUnit.getCalcType() == CalcType.INDEX){
                    j.getAndIncrement();
                    indexResult.putAll(result);
                }
            }
        });
        StringBuffer sb = new StringBuffer();
        sourceResults.forEach((calcUnit,result) -> {
            if (i.get() > 0 && j.get() == 0){
                // 如果数据集的数量大于0
                String logic = indexConfigModel.getIndexLogic();
                // 执行sql，并放进结果集内
                thisResult.put(this.getCalcCode(),DBExecutor.doQueryMap(logic));
            }
            if(j.get() > 0 && i.get() == 0){
                // 如果指标的数量大于0
                String logic = indexConfigModel.getIndexLogic();
                // 执行模板，结果放入结果集
                ScriptUtil.executeScript(indexConfigModel.getIndexNo(),logic,result);
            }

        });
        if (i.get() > 0 && j.get() >0) {
            // 同时具有指标和数据集
            String logic = indexConfigModel.getIndexLogic();
            // 先将sql作为一个模板进行处理，将指标的结果作为参数带入模板成为一个新模板，后在将数据集的参数带入
            String editSql = (String) ScriptUtil.executeScript(indexConfigModel.getIndexNo(),logic,indexResult);
            thisResult.put(this.getCalcCode(),DBExecutor.doQueryMap(logic,dataSetResult));
        }



//        Map<CalcUnit, CalcResult<String, Object>> dataMap = new HashMap<>();
//
//        // 获取 数据源 A 配置的 用户
//        // 获取 数据源 B 配置的 业务
//        sourceResults.forEach((calcUnit, result) -> {
//            if (calcUnit.getCalcCode().equals("data")) {
//                dataMap.put(calcUnit, result);
//            }
//        });
//
//        // 计算 年龄小于50的用户数量 X 指标
//        // 计算 价格大于 500 的业务数量 Y 指标
//        dataMap.forEach((calcUnit, result) -> {
//            if (calcUnit.getCalcCode().equals("data")) {
//                List<User> userList = (ArrayList<User>) result.get("data-user");
//                long userCount = userList.stream().filter(user -> user.age()<50).count();
//                thisResult.put(this.getCalcCode()+"-userCount",userCount);
//            }
//            if (calcUnit.getCalcCode().equals("data")) {
//                List<Bussiness> bussinesseList = (ArrayList<Bussiness>) result.get("data-buss");
//                long bussCount = bussinesseList.stream().filter(bussiness -> bussiness.price()>500).count();
//                thisResult.put(this.getCalcCode()+"-bussCount",bussCount);
//            }
//        });
    }
}
