package com.sundata.internalevaluation.script.execute;

import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import groovy.lang.Script;
import org.apache.commons.pool2.ObjectPool;
import com.sundata.internalevaluation.script.cache.ScriptLRUCache;
import com.sundata.internalevaluation.script.model.ScriptModel;
import com.sundata.internalevaluation.script.pool.GroovyScriptPool;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Map;
import java.util.concurrent.atomic.AtomicReference;

public class ScriptExecute {
    private static final Logger log = LoggerFactory.getLogger(ScriptExecute.class);
    private static final ScriptLRUCache<ScriptModel, ObjectPool<Script>> cache = new ScriptLRUCache<>(200);

    private static void addCache(ScriptModel model, ObjectPool<Script> pool) {
        cache.put(model, pool);
    }

    public static void addCache(String scriptId, String scriptStr) {
        ScriptModel model = new ScriptModel(scriptId, scriptStr);
        ObjectPool<Script> pool = GroovyScriptPool.createGroovyShellPool(model);
        addCache(model, pool);
    }

    /**
     * 根据数据对象对缓存进行处理，使用一次就处理一次
     *
     * @param model 脚本对象
     * @return 池对象
     */
    public static ObjectPool<Script> getPool(ScriptModel model) {
        if (cache.containsKey(model)) {
            return cache.get(model);
        } else {
            ObjectPool<Script> pool = GroovyScriptPool.createGroovyShellPool(model);
            addCache(model, pool);
            return cache.get(model);
        }
    }

    /**
     * 根据ID获取缓存中的数据内容，不建议直接使用，需要确保数据中存在才行
     *
     * @param scriptId 脚本ID
     * @return 脚本池
     */
    public static ObjectPool<Script> getPoolById(String scriptId) {
        AtomicReference<ObjectPool<Script>> pool = new AtomicReference<>();
        cache.forEach((c, p) -> {
            if (scriptId.equals(c.getScriptId())) {
                pool.set(p);
            }
        });
        if (pool.get() != null) {
            return pool.get();
        } else {
            // 缓存级数据没有就没有，没有需要重新构建
            return null;
        }
    }

    /**
     * 根据ID获取缓存中的数据内容，不建议直接使用，需要确保数据中存在才行
     *
     * @param scriptStr 脚本内容
     * @return 脚本池
     */
    public static ObjectPool<Script> getPoolByScriptStr(String scriptStr) {
        AtomicReference<ObjectPool<Script>> pool = new AtomicReference<>();
        cache.forEach((c, p) -> {
            if (scriptStr.equals(c.getScriptStr())) {
                pool.set(p);
            }
        });
        if (pool.get() != null) {
            return pool.get();
        } else {
            // 缓存级数据没有就没有，没有需要重新构建
            return null;
        }
    }

    public static boolean hasCacheById(String scriptId) {
        return getPoolById(scriptId) != null;
    }
    public static boolean hasCacheByScriptStr(String scriptStr) {
        return getPoolByScriptStr(scriptStr) != null;
    }

    public static boolean hasCache(String scriptId, String scriptStr) {
        return getPool(new ScriptModel(scriptId, scriptStr)) != null;
    }

    /**
     * 计算主方法
     *
     * @param scriptId  计算id
     * @param scriptStr 脚本类
     * @param params 脚本参数
     * @return 脚本计算结果
     */
    public static Object executeScript(String scriptId, String scriptStr, Map<String, Object> params) {
        AtomicReference<ScriptModel> model = new AtomicReference<>();
        if (StrUtil.isNotBlank(scriptId) && hasCacheById(scriptId)) {
            String finalScriptId = scriptId;
            cache.keySet().forEach((k) -> {
                if (k.getScriptId().equals(finalScriptId)) {
                    model.set(k);
                }
            });
            return executeScript(model.get(), params);
        }else if(StrUtil.isNotBlank(scriptStr) && hasCacheByScriptStr(scriptStr) ) {
            cache.keySet().forEach((k) -> {
                if (k.getScriptStr().equals(scriptStr)) {
                    model.set(k);
                }
            });
            return executeScript(model.get(), params);
        } else if (hasCache(scriptId, scriptStr) && StrUtil.isNotBlank(scriptStr)) {
            return executeScript(new ScriptModel(scriptId, scriptStr), params);
        } else {
            if (StrUtil.isBlank(scriptStr) && StrUtil.isBlank(scriptId)) {
                throw new NullPointerException("脚本与id均为空，无法执行计算");
            }
            if (StrUtil.isBlank(scriptId)) {
                scriptId = RandomUtil.randomString(10);
            }
            return executeScript(new ScriptModel(scriptId, scriptStr), params);
        }


    }

    /**
     * 计算的基础方法
     *
     * @param scriptModel 对象清单
     * @param params      计算用脚本参数
     * @return 计算结果
     */
    private static Object executeScript(ScriptModel scriptModel, Map<String, Object> params) {
        if (scriptModel == null) {
            throw new NullPointerException("scriptModel is null");
        }
        ObjectPool<Script> pool = getPool(scriptModel);
        try {
            Script script = pool.borrowObject();
            if (params != null) {
                params.forEach((k, v) -> script.getBinding().setVariable(k, v));
            }
            return script.run();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        throw new RuntimeException("计算失败，请检查实际执行情况与日志！");
    }
}
